==> datetime_flatpickr-2.0.7.tar.gz: datetime_flatpickr/src/Plugin/Field/FieldWidget/DateTimeFlatPickrWidgetTrait.php <==
<?php

namespace Drupal\datetime_flatpickr\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItem;
use Drupal\Component\Utility\Html;

/**
 * Trait of Plugin implementation of the various Flatpickr datetime widgets.
 *
 * @var Trait
 * @see \Drupal\datetime_flatpickr\Plugin\Field\FieldWidget\DateTimeFlatPickrWidget
 * @see \Drupal\datetime_flatpickr\Plugin\Field\FieldWidget\DateTimeRangeFlatPickrWidget
 */
trait DateTimeFlatPickrWidgetTrait {

  /**
   * Array of  available languages.
   *
   * @var string[]
   */
  protected static $availableLanguages = [
    'ar',
    'at',
    'az',
    'be',
    'bg',
    'bn',
    'bs',
    'at',
    'cs',
    'cy',
    'da',
    'de',
    'eo',
    'es',
    'et',
    'fa',
    'fi',
    'fo',
    'fr',
    'ga',
    'gr',
    'he',
    'hi',
    'hr',
    'hu',
    'id',
    'is',
    'it',
    'ja',
    'ka',
    'km',
    'ko',
    'kz',
    'lt',
    'lv',
    'mk',
    'mn',
    'ms',
    'my',
    'nl',
    'no',
    'pa',
    'pl',
    'pt',
    'ro',
    'ru',
    'si',
    'sk',
    'sl',
    'sq',
    'sr',
    'sv',
    'th',
    'tr',
    'uk',
    'vn',
    'zh',
  ];

  /**
   * {@inheritdoc}
   */
  protected function createDefaultValue($date, $timezone) {
    // The date was created and verified during field_load(), so it is safe to
    // use without further inspection.
    if ($this->getFieldSetting('datetime_type') === DateTimeItem::DATETIME_TYPE_DATE) {
      $date->setDefaultDateTime();
    }
    $date->setTimezone(new \DateTimeZone($timezone));
    $date->render();
    return $date->render();
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      // Exactly the same as date format, but for the altInput field.
      'altFormat' => 'F j, Y',
      // Show the user a readable date (as per altFormat), but return something
      // totally different to the server.
      'altInput' => FALSE,
      // A string of characters which are used to define how the date will be
      // displayed in the input box. The supported characters are defined in
      // the table below.
      'dateFormat' => 'Y-m-d H:i',
      // Sets the initial selected date(s). If you're using mode: "multiple"
      // or a range calendar supply an Array of Date objects or an Array of
      // date strings which follow your dateFormat. Otherwise, you can supply
      // a single Date object or a date string.
      'defaultDate' => '',
      // Initial value of the hour element.
      'defaultHour' => '',
      // Initial value of the minute element.
      'defaultMinute' => '',
      // Enables time picker.
      'enableTime' => FALSE,
      // Enables seconds in the time picker.
      'enableSeconds' => FALSE,
      // Displays the calendar inline.
      'inline' => FALSE,
      // The maximum date that a user can pick to (inclusive).
      'maxDate' => '',
      // The minimum date that a user can start picking from (inclusive).
      'minDate' => '',
      // The maximum time that a user can pick to (inclusive).
      'maxTime' => ['hour' => '', 'min' => ''],
      // The minimum time that a user can start picking from (inclusive).
      'minTime' => ['hour' => '', 'min' => ''],
      // Adjusts the step for the minute input (incl. scrolling)
      'minuteIncrement' => 5,
      // Where the calendar is rendered relative to the input.
      // "auto", "above" or "below".
      'position' => 'auto',
      // Displays time picker in 24 hour mode without AM/PM
      // selection when enabled.
      'time_24hr' => FALSE,
      // Enables display of week numbers in calendar.
      'weekNumbers' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);
    $options = $this->getSettings();

    $element['altInput'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Alternative input'),
      '#description' => $this->t('Show the user a readable date (as per altFormat), but return something totally different to the server.'),
      '#default_value' => $options['altInput'],
    ];
    $element['altFormat'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Alternative format'),
      '#description' => $this->t('Exactly the same as date format, but for the altInput field.'),
      '#default_value' => $options['altFormat'],
      '#states' => [
        'visible' => [
          'input[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][altInput]"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $element['dateFormat'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Date format'),
      '#description' => $this->t("A string of characters which are used to define how the date will be displayed in the input box. The supported characters are defined in the table below."),
      '#default_value' => $options['dateFormat'],
    ];
    $element['enableTime'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enables time picker'),
      '#default_value' => $options['enableTime'],
    ];
    $element['enableSeconds'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enables seconds'),
      '#description' => $this->t('Enables seconds in the time picker.'),
      '#default_value' => $options['enableSeconds'],
    ];
    $element['inline'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Inline calendar'),
      '#description' => $this->t('Displays the calendar inline.'),
      '#default_value' => $options['inline'],
    ];
    $element['minDate'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Minimum date'),
      '#description' => $this->t('The minimum date that a user can start picking from (inclusive).'),
      '#default_value' => $options['minDate'],
    ];
    $element['maxDate'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Maximum date'),
      '#description' => $this->t('The maximum date that a user can pick to (inclusive).'),
      '#default_value' => $options['maxDate'],
    ];
    $element['minTime'] = [
      '#type' => 'details',
      '#title' => $this->t('Minimum time'),
      '#description' => $this->t('The minimum time that a user can start picking from (inclusive).'),
      '#open' => FALSE,
      'hour' => [
        '#type' => 'textfield',
        '#title' => $this->t('Hour'),
        '#default_value' => $options['minTime']['hour'],
        '#element_validate' => [
          [$this, 'fieldSettingsHourElementValidate'],
        ],
      ],
      'min' => [
        '#type' => 'textfield',
        '#title' => $this->t('Minute'),
        '#default_value' => $options['minTime']['min'],
        '#element_validate' => [
          [$this, 'fieldSettingsMinElementValidate'],
        ],
      ],
    ];
    $element['maxTime'] = [
      '#type' => 'details',
      '#title' => $this->t('Maximum time'),
      '#description' => $this->t('The maximum time that a user can pick to (inclusive).'),
      '#open' => FALSE,
      'hour' => [
        '#type' => 'textfield',
        '#title' => $this->t('Hour'),
        '#default_value' => $options['maxTime']['hour'],
        '#element_validate' => [
          [$this, 'fieldSettingsHourElementValidate'],
        ],
      ],
      'min' => [
        '#type' => 'textfield',
        '#title' => $this->t('Minute'),
        '#default_value' => $options['maxTime']['min'],
        '#element_validate' => [
          [$this, 'fieldSettingsMinElementValidate'],
        ],
      ],
    ];
    $element['minuteIncrement'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Minute increment'),
      '#description' => $this->t('Adjusts the step for the minute input (incl. scrolling).'),
      '#default_value' => $options['minuteIncrement'],
    ];
    $element['position'] = [
      '#type' => 'select',
      '#title' => $this->t('Position'),
      '#default_value' => $options['position'],
      '#description' => $this->t('Where the calendar is rendered relative to the input.'),
      '#options' => [
        'auto' => $this->t('Auto'),
        'above' => $this->t('Above'),
        'below' => $this->t('Below'),
      ],
    ];
    $element['time_24hr'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Time 24h'),
      '#description' => $this->t('Displays time picker in 24 hour mode without AM/PM selection when enabled.'),
      '#default_value' => $options['time_24hr'],
    ];
    $element['weekNumbers'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show week numbers'),
      '#description' => $this->t('Enables display of week numbers in calendar.'),
      '#default_value' => $options['weekNumbers'],
    ];

    return $element;
  }

  /**
   * Hour element validation.
   */
  public function fieldSettingsHourElementValidate(&$element, FormStateInterface $form_state) {
    $setting = &$form_state->getValue($element['#parents']);
    if (isset($setting)) {
      // For two-tiered array.
      $limits = [0, 23];
      // Validate int hours and minutes settings.
      if ($setting !== '') {
        if (!is_numeric($setting) || intval($setting) != $setting || $setting < $limits[0] || $setting > $limits[1]) {
          $t_args = [
            '%name' => $element['#title'],
            '@start' => $limits[0],
            '@end' => $limits[1],
          ];
          $form_state->setError($element, $this->t('%name must be an integer between @start and @end.', $t_args));
        }
      }
    }
  }

  /**
   * Minute element validation.
   */
  public function fieldSettingsMinElementValidate(&$element, FormStateInterface $form_state) {
    $setting = &$form_state->getValue($element['#parents']);
    if (isset($setting)) {
      // For two-tiered array.
      $limits = [0, 59];
      // Validate int hours and minutes settings.
      if ($setting !== '') {
        if (!is_numeric($setting) || intval($setting) != $setting || $setting < $limits[0] || $setting > $limits[1]) {
          $t_args = [
            '%name' => $element['#title'],
            '@start' => $limits[0],
            '@end' => $limits[1],
          ];
          $form_state->setError($element, $this->t('%name must be an integer between @start and @end.', $t_args));
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $options = $this->getSettings();

    if ($options['altInput']) {
      $summary[] = $this->t('Alternative input format: @alt_input', ['@alt_input' => $options['altFormat']]);
    }
    $summary[] = $this->t('Date format: @date_format', ['@date_format' => $options['dateFormat']]);

    if ($options['enableTime']) {
      $summary[] = $this->t('Enabled time picker.');
    }

    if ($options['enableSeconds']) {
      $summary[] = $this->t('Enabled seconds.');
    }

    if ($options['inline']) {
      $summary[] = $this->t('Inline calendar.');
    }

    if ($options['minDate']) {
      $summary[] = $this->t('Min date: @min_date', ['@min_date' => $options['minDate']]);
    }

    if ($options['maxDate']) {
      $summary[] = $this->t('Max date: @max_date', ['@max_date' => $options['maxDate']]);
    }

    if ($options['minTime']['hour'] || $options['minTime']['min']) {
      $summary[] = $this->t('Min time: @hour:@min', [
        '@hour' => $options['minTime']['hour'],
        '@min' => $options['minTime']['min'],
      ]);
    }

    if ($options['maxTime']['hour'] || $options['maxTime']['min']) {
      $summary[] = $this->t('Max time: @hour:@min', [
        '@hour' => $options['maxTime']['hour'],
        '@min' => $options['maxTime']['min'],
      ]);
    }

    $summary[] = $this->t('Minute increment: @inc', ['@inc' => $options['minuteIncrement']]);
    $summary[] = $this->t('Position: @pos', ['@pos' => $options['position']]);

    if ($options['time_24hr']) {
      $summary[] = $this->t('Use 24h time');
    }

    if ($options['weekNumbers']) {
      $summary[] = $this->t('Show week numbers');
    }

    return $summary;
  }

  /**
   * Function of typification options Timepicker.
   *
   * @param array $settings
   *   Settings for JS Timepicker.
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   The Field items.
   *
   * @return array
   *   return array of changed settings after typefications of all parameters.
   */
  public static function processFieldSettings(array $settings, FieldItemListInterface $items) {
    $options = $settings ?? [];

    if (!empty($options)) {
      $groups = [
        'boolean' => [
          'altInput',
          'enableTime',
          'enableSeconds',
          'inline',
          'time_24hr',
          'weekNumbers',
        ],
        'int' => [
          'minTime',
          'maxTime',
          'minuteIncrement',
        ],
        'no_filtering' => [],
      ];
      // Callback for the array_walk_recursive().
      $filter = function (&$item, $key, $groups) {
        if (in_array($key, $groups['boolean'], TRUE)) {
          if ($item !== NULL) {
            $item = (bool) $item;
          }
        }
        elseif (in_array($key, $groups['int'], TRUE)) {
          if ($item !== NULL) {
            if (is_array($item)) {
              foreach ($item as $sub_key => $sub_item) {
                if ($item !== NULL) {
                  $item[$sub_key] = (int) $sub_item;
                }
              }
            }
            else {
              $item = (int) $item;
            }
          }
        }
        elseif (in_array($key, $groups['no_filtering'], TRUE)) {
          // Do nothing.
        }
        else {
          // @todo Use filter_xss_admin() instead?
          $item = Html::escape($item);
        }
      };
      // Filter user submitted settings since plugin builds output by just
      // concatenation of strings so it's possible, for example,
      // to insert html into labels.
      array_walk_recursive($options, $filter, $groups);

      $options['dateFormat'] = str_replace('s', 'S', $options['dateFormat']);
      if (isset($options['altFormat'])) {
        $options['altFormat'] = str_replace('s', 'S', $options['altFormat']);
      }
    }

    $lang_code = $items->getLangcode();
    if (in_array($lang_code, static::$availableLanguages)) {
      $options['locale'] = mb_strtolower($lang_code);
    }

    return static::fieldSettingsFinalNullCleanType($options);
  }

  /**
   * Method deleting Null parameters before send to JS.
   *
   * @param array $settings
   *   Non-filter parameters.
   *
   * @return array
   *   Returned filtering Parameters for send to JS.
   */
  public static function fieldSettingsFinalNullCleanType(array &$settings) {
    // Convert boolean settings to boolean.
    $boolean = [
      'altInput',
      'enableTime',
      'enableSeconds',
      'inline',
      'time_24hr',
      'weekNumbers',
    ];
    foreach ($boolean as $key) {
      if (!empty($settings[$key])) {
        $new[$key] = (bool) $settings[$key];
      }
    }

    if (isset($settings['minTime'])) {
      if ('' !== $settings['minTime']['hour'] && '' !== $settings['minTime']['min']) {
        $new['minTime'] = $settings['minTime']['hour'] . ':' . $settings['minTime']['min'];
      }
    }

    if (isset($settings['maxTime'])) {
      if (!empty($settings['maxTime']['hour']) && !empty($settings['maxTime']['min'])) {
        $new['maxTime'] = $settings['maxTime']['hour'] . ':' . $settings['maxTime']['min'];
      }
    }

    $format_array = [
      'altFormat',
      'dateFormat',
      'minDate',
      'maxDate',
      'minuteIncrement',
      'position',
      'locale',
    ];
    foreach ($format_array as $key) {
      if (!empty($settings[$key])) {
        $new[$key] = $settings[$key];
      }
    }

    if (isset($settings['altInput']) && !$settings['altInput']) {
      unset($new['altInput'], $new['altFormat']);
    }

    return $new;
  }

}

==> datetime_flatpickr-2.0.8.tar.gz: datetime_flatpickr/src/Plugin/Field/FieldWidget/DateTimeFlatPickrWidgetTrait.php <==
<?php

namespace Drupal\datetime_flatpickr\Plugin\Field\FieldWidget;

use Drupal\Core\Datetime\DateHelper;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\Checkboxes;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItem;
use Drupal\Component\Utility\Html;

/**
 * Trait of Plugin implementation of the various Flatpickr datetime widgets.
 *
 * @var Trait
 * @see \Drupal\datetime_flatpickr\Plugin\Field\FieldWidget\DateTimeFlatPickrWidget
 * @see \Drupal\datetime_flatpickr\Plugin\Field\FieldWidget\DateTimeRangeFlatPickrWidget
 */
trait DateTimeFlatPickrWidgetTrait {

  /**
   * Array of  available languages.
   *
   * @var string[]
   */
  protected static $availableLanguages = [
    'ar',
    'at',
    'az',
    'be',
    'bg',
    'bn',
    'bs',
    'at',
    'cs',
    'cy',
    'da',
    'de',
    'eo',
    'es',
    'et',
    'fa',
    'fi',
    'fo',
    'fr',
    'ga',
    'gr',
    'he',
    'hi',
    'hr',
    'hu',
    'id',
    'is',
    'it',
    'ja',
    'ka',
    'km',
    'ko',
    'kz',
    'lt',
    'lv',
    'mk',
    'mn',
    'ms',
    'my',
    'nl',
    'no',
    'pa',
    'pl',
    'pt',
    'ro',
    'ru',
    'si',
    'sk',
    'sl',
    'sq',
    'sr',
    'sv',
    'th',
    'tr',
    'uk',
    'vn',
    'zh',
  ];

  /**
   * {@inheritdoc}
   */
  protected function createDefaultValue($date, $timezone) {
    // The date was created and verified during field_load(), so it is safe to
    // use without further inspection.
    if ($this->getFieldSetting('datetime_type') === DateTimeItem::DATETIME_TYPE_DATE) {
      $date->setDefaultDateTime();
    }
    $date->setTimezone(new \DateTimeZone($timezone));
    $date->render();
    return $date->render();
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      // Exactly the same as date format, but for the altInput field.
      'altFormat' => 'F j, Y',
      // Show the user a readable date (as per altFormat), but return something
      // totally different to the server.
      'altInput' => FALSE,
      // A string of characters which are used to define how the date will be
      // displayed in the input box. The supported characters are defined in
      // the table below.
      'dateFormat' => 'Y-m-d H:i',
      // Sets the initial selected date(s). If you're using mode: "multiple"
      // or a range calendar supply an Array of Date objects or an Array of
      // date strings which follow your dateFormat. Otherwise, you can supply
      // a single Date object or a date string.
      'defaultDate' => '',
      // Initial value of the hour element.
      'defaultHour' => '',
      // Initial value of the minute element.
      'defaultMinute' => '',
      // Enables time picker.
      'enableTime' => FALSE,
      // Enables seconds in the time picker.
      'enableSeconds' => FALSE,
      // Displays the calendar inline.
      'inline' => FALSE,
      // The maximum date that a user can pick to (inclusive).
      'maxDate' => '',
      // The minimum date that a user can start picking from (inclusive).
      'minDate' => '',
      // The maximum time that a user can pick to (inclusive).
      'maxTime' => ['hour' => '', 'min' => ''],
      // The minimum time that a user can start picking from (inclusive).
      'minTime' => ['hour' => '', 'min' => ''],
      // Adjusts the step for the minute input (incl. scrolling)
      'minuteIncrement' => 5,
      // Where the calendar is rendered relative to the input.
      // "auto", "above" or "below".
      'position' => 'auto',
      // Displays time picker in 24 hour mode without AM/PM
      // selection when enabled.
      'time_24hr' => FALSE,
      // Enables display of week numbers in calendar.
      'weekNumbers' => FALSE,
      // The days of the week for disabling from calendar.
      'disabledWeekDays' => [],
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);
    $options = $this->getSettings();

    $element['altInput'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Alternative input'),
      '#description' => $this->t('Show the user a readable date (as per altFormat), but return something totally different to the server.'),
      '#default_value' => $options['altInput'],
    ];
    $element['altFormat'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Alternative format'),
      '#description' => $this->t('Exactly the same as date format, but for the altInput field.'),
      '#default_value' => $options['altFormat'],
      '#states' => [
        'visible' => [
          'input[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][altInput]"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $element['dateFormat'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Date format'),
      '#description' => $this->t("A string of characters which are used to define how the date will be displayed in the input box. The supported characters are defined in the table below."),
      '#default_value' => $options['dateFormat'],
    ];
    $element['enableTime'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enables time picker'),
      '#default_value' => $options['enableTime'],
    ];
    $element['enableSeconds'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enables seconds'),
      '#description' => $this->t('Enables seconds in the time picker.'),
      '#default_value' => $options['enableSeconds'],
    ];
    $element['inline'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Inline calendar'),
      '#description' => $this->t('Displays the calendar inline.'),
      '#default_value' => $options['inline'],
    ];
    $element['minDate'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Minimum date'),
      '#description' => $this->t('The minimum date that a user can start picking from (inclusive).'),
      '#default_value' => $options['minDate'],
    ];
    $element['maxDate'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Maximum date'),
      '#description' => $this->t('The maximum date that a user can pick to (inclusive).'),
      '#default_value' => $options['maxDate'],
    ];
    $element['minTime'] = [
      '#type' => 'details',
      '#title' => $this->t('Minimum time'),
      '#description' => $this->t('The minimum time that a user can start picking from (inclusive).'),
      '#open' => FALSE,
      'hour' => [
        '#type' => 'textfield',
        '#title' => $this->t('Hour'),
        '#default_value' => $options['minTime']['hour'],
        '#element_validate' => [
          [$this, 'fieldSettingsHourElementValidate'],
        ],
      ],
      'min' => [
        '#type' => 'textfield',
        '#title' => $this->t('Minute'),
        '#default_value' => $options['minTime']['min'],
        '#element_validate' => [
          [$this, 'fieldSettingsMinElementValidate'],
        ],
      ],
    ];
    $element['maxTime'] = [
      '#type' => 'details',
      '#title' => $this->t('Maximum time'),
      '#description' => $this->t('The maximum time that a user can pick to (inclusive).'),
      '#open' => FALSE,
      'hour' => [
        '#type' => 'textfield',
        '#title' => $this->t('Hour'),
        '#default_value' => $options['maxTime']['hour'],
        '#element_validate' => [
          [$this, 'fieldSettingsHourElementValidate'],
        ],
      ],
      'min' => [
        '#type' => 'textfield',
        '#title' => $this->t('Minute'),
        '#default_value' => $options['maxTime']['min'],
        '#element_validate' => [
          [$this, 'fieldSettingsMinElementValidate'],
        ],
      ],
    ];
    $element['minuteIncrement'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Minute increment'),
      '#description' => $this->t('Adjusts the step for the minute input (incl. scrolling).'),
      '#default_value' => $options['minuteIncrement'],
    ];
    $element['position'] = [
      '#type' => 'select',
      '#title' => $this->t('Position'),
      '#default_value' => $options['position'],
      '#description' => $this->t('Where the calendar is rendered relative to the input.'),
      '#options' => [
        'auto' => $this->t('Auto'),
        'above' => $this->t('Above'),
        'below' => $this->t('Below'),
      ],
    ];
    $element['time_24hr'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Time 24h'),
      '#description' => $this->t('Displays time picker in 24 hour mode without AM/PM selection when enabled.'),
      '#default_value' => $options['time_24hr'],
    ];
    $element['weekNumbers'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show week numbers'),
      '#description' => $this->t('Enables display of week numbers in calendar.'),
      '#default_value' => $options['weekNumbers'],
    ];

    $element['disabledWeekDays'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Disabled week days'),
      '#description' => $this->t('Select the days of the week for disabling from calendar.'),
      '#options' =>  DateHelper::weekDays(TRUE),
      '#default_value' => Checkboxes::getCheckedCheckboxes($options['disabledWeekDays']),
    ];

    return $element;
  }

  /**
   * Hour element validation.
   */
  public function fieldSettingsHourElementValidate(&$element, FormStateInterface $form_state) {
    $setting = &$form_state->getValue($element['#parents']);
    if (isset($setting)) {
      // For two-tiered array.
      $limits = [0, 23];
      // Validate int hours and minutes settings.
      if ($setting !== '') {
        if (!is_numeric($setting) || intval($setting) != $setting || $setting < $limits[0] || $setting > $limits[1]) {
          $t_args = [
            '%name' => $element['#title'],
            '@start' => $limits[0],
            '@end' => $limits[1],
          ];
          $form_state->setError($element, $this->t('%name must be an integer between @start and @end.', $t_args));
        }
      }
    }
  }

  /**
   * Minute element validation.
   */
  public function fieldSettingsMinElementValidate(&$element, FormStateInterface $form_state) {
    $setting = &$form_state->getValue($element['#parents']);
    if (isset($setting)) {
      // For two-tiered array.
      $limits = [0, 59];
      // Validate int hours and minutes settings.
      if ($setting !== '') {
        if (!is_numeric($setting) || intval($setting) != $setting || $setting < $limits[0] || $setting > $limits[1]) {
          $t_args = [
            '%name' => $element['#title'],
            '@start' => $limits[0],
            '@end' => $limits[1],
          ];
          $form_state->setError($element, $this->t('%name must be an integer between @start and @end.', $t_args));
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $options = $this->getSettings();

    if ($options['altInput']) {
      $summary[] = $this->t('Alternative input format: @alt_input', ['@alt_input' => $options['altFormat']]);
    }
    $summary[] = $this->t('Date format: @date_format', ['@date_format' => $options['dateFormat']]);

    if ($options['enableTime']) {
      $summary[] = $this->t('Enabled time picker.');
    }

    if ($options['enableSeconds']) {
      $summary[] = $this->t('Enabled seconds.');
    }

    if ($options['inline']) {
      $summary[] = $this->t('Inline calendar.');
    }

    if ($options['minDate']) {
      $summary[] = $this->t('Min date: @min_date', ['@min_date' => $options['minDate']]);
    }

    if ($options['maxDate']) {
      $summary[] = $this->t('Max date: @max_date', ['@max_date' => $options['maxDate']]);
    }

    if ($options['minTime']['hour'] || $options['minTime']['min']) {
      $summary[] = $this->t('Min time: @hour:@min', [
        '@hour' => $options['minTime']['hour'],
        '@min' => $options['minTime']['min'],
      ]);
    }

    if ($options['maxTime']['hour'] || $options['maxTime']['min']) {
      $summary[] = $this->t('Max time: @hour:@min', [
        '@hour' => $options['maxTime']['hour'],
        '@min' => $options['maxTime']['min'],
      ]);
    }

    $summary[] = $this->t('Minute increment: @inc', ['@inc' => $options['minuteIncrement']]);
    $summary[] = $this->t('Position: @pos', ['@pos' => $options['position']]);

    if ($options['time_24hr']) {
      $summary[] = $this->t('Use 24h time');
    }

    if ($options['weekNumbers']) {
      $summary[] = $this->t('Show week numbers');
    }

    if ($options['disabledWeekDays']) {
      $disabledWeekDays = implode(', ', array_intersect_key(DateHelper::weekDays(TRUE), array_flip(Checkboxes::getCheckedCheckboxes($options['disabledWeekDays']))));
      $summary[] = $this->t('Disabled week days: @disabledWeekDays', ['@disabledWeekDays' => $disabledWeekDays]);
    }

    return $summary;
  }

  /**
   * Function of typification options Timepicker.
   *
   * @param array $settings
   *   Settings for JS Timepicker.
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   The Field items.
   *
   * @return array
   *   return array of changed settings after typefications of all parameters.
   */
  public static function processFieldSettings(array $settings, FieldItemListInterface $items) {
    $options = $settings ?? [];

    if (!empty($options)) {
      $groups = [
        'boolean' => [
          'altInput',
          'enableTime',
          'enableSeconds',
          'inline',
          'time_24hr',
          'weekNumbers',
        ],
        'int' => [
          'minTime',
          'maxTime',
          'minuteIncrement',
        ],
        'no_filtering' => [],
      ];
      // Callback for the array_walk().
      $filter = function (&$item, $key, $groups) {
        if (in_array($key, $groups['boolean'], TRUE)) {
          if ($item !== NULL) {
            $item = (bool) $item;
          }
        }
        elseif (in_array($key, $groups['int'], TRUE)) {
          if ($item !== NULL) {
            if (is_array($item)) {
              foreach ($item as $sub_key => $sub_item) {
                if ($item !== NULL) {
                  $item[$sub_key] = (int) $sub_item;
                }
              }
            }
            else {
              $item = (int) $item;
            }
          }
        }
        elseif (is_array($item) || in_array($key, $groups['no_filtering'], TRUE)) {
          // Do nothing.
        }
        else {
          // @todo Use filter_xss_admin() instead?
          $item = Html::escape($item);
        }
      };
      // Filter user submitted settings since plugin builds output by just
      // concatenation of strings so it's possible, for example,
      // to insert html into labels.
      array_walk($options, $filter, $groups);

      $options['dateFormat'] = str_replace('s', 'S', $options['dateFormat']);
      if (isset($options['altFormat'])) {
        $options['altFormat'] = str_replace('s', 'S', $options['altFormat']);
      }
    }

    $lang_code = $items->getLangcode();
    if (in_array($lang_code, static::$availableLanguages)) {
      $options['locale'] = mb_strtolower($lang_code);
    }

    return static::fieldSettingsFinalNullCleanType($options);
  }

  /**
   * Method deleting Null parameters before send to JS.
   *
   * @param array $settings
   *   Non-filter parameters.
   *
   * @return array
   *   Returned filtering Parameters for send to JS.
   */
  public static function fieldSettingsFinalNullCleanType(array &$settings) {
    // Convert boolean settings to boolean.
    $boolean = [
      'altInput',
      'enableTime',
      'enableSeconds',
      'inline',
      'time_24hr',
      'weekNumbers',
    ];
    foreach ($boolean as $key) {
      if (!empty($settings[$key])) {
        $new[$key] = (bool) $settings[$key];
      }
    }

    if (isset($settings['minTime'])) {
      if ('' !== $settings['minTime']['hour'] && '' !== $settings['minTime']['min']) {
        $new['minTime'] = $settings['minTime']['hour'] . ':' . $settings['minTime']['min'];
      }
    }

    if (isset($settings['maxTime'])) {
      if (!empty($settings['maxTime']['hour']) && !empty($settings['maxTime']['min'])) {
        $new['maxTime'] = $settings['maxTime']['hour'] . ':' . $settings['maxTime']['min'];
      }
    }

    $format_array = [
      'altFormat',
      'dateFormat',
      'minDate',
      'maxDate',
      'minuteIncrement',
      'position',
      'locale',
    ];

    foreach ($format_array as $key) {
      if (!empty($settings[$key])) {
        $new[$key] = $settings[$key];
      }
    }

    // Special case for Disabled week days, we need an array of numbers.
    $new['disabledWeekDays'] = Checkboxes::getCheckedCheckboxes($settings['disabledWeekDays']);

    if (isset($settings['altInput']) && !$settings['altInput']) {
      unset($new['altInput'], $new['altFormat']);
    }

    return $new;
  }

}
